<?php
/**
*
* @package Scheduled Group Membership
* @version $Id: functions_groups_scheduling.php 2008-12-29 DavidIQ $
* @copyright (c) 2009 DavidIQ.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Add sgm columns to SQL query
*/
function sgm_sql_add_cols($sql, $table_alias = '')
{
	//Add table alias
	$ta = empty($table_alias) ? '' : $table_alias . '.';
	$sgm_cols = " {$ta}group_schedule_days, {$ta}group_schedule_start, {$ta}group_schedule_end, ";
	
	return substr_replace($sql, $sgm_cols, strpos($sql, ' '), 1);
}

/**
* Assign the template variables for admin
*/
function sgm_acp_template_append(&$template, $data, $block_name)
{
	$template->alter_block_array($block_name, array(
		'S_GROUP_SCHEDULED'			=> ($data['group_schedule_start'] > 0) ? true : false,
		'SCHEDULE_DAYS'				=> schedule_group_days_convert(unserialize($data['group_schedule_days'])),
		'SCHEDULE_START'			=> schedule_group_time_convert($data['group_schedule_start']),
		'SCHEDULE_END'				=> schedule_group_time_convert($data['group_schedule_end'])), true, 'change');
}

/**
* Assign the template variables outside of admin
*/
function sgm_template_append(&$template, $data, $block_name)
{
	$template->alter_block_array($block_name, array(
		'S_GROUP_SCHEDULED'			=> ($data['group_schedule_start'] > 0) ? true : false,
		'SCHEDULE_DAYS'				=> schedule_group_days_convert(unserialize($data['group_schedule_days'])),
		'SCHEDULE_START'			=> schedule_group_time_convert($data['group_schedule_start']),
		'SCHEDULE_END'				=> schedule_group_time_convert($data['group_schedule_end'])), true, 'change');
}

/**
* Check if a user has a scheduled group membership
*/
function user_has_sgm($user_id)
{
	global $db;
	
	$sql = 'SELECT group_schedule_days, group_schedule_start, group_schedule_end
			FROM ' . USER_GROUP_TABLE . "
			WHERE user_id = $user_id";
	$result = $db->sql_query($sql);

	$has_sgm = false;
	while ($row = $db->sql_fetchrow($result))
	{
		if (isset($row['group_schedule_start']) && !empty($row['group_schedule_start']))
		{
			$has_sgm = true;
			break;
		}
	}
	
	$db->sql_freeresult($result);
	
	return $has_sgm;
}

/**
* Check for group membership scheduling
*/
function group_membership_scheduled($schedule_days, $schedule_start, $schedule_end)
{
	global $config;
	
	//Not a scheduled membership
	if (empty($schedule_days) && $schedule_start == 0 && $schedule_end == 0)
	{
		return true;
	}
	
	$days = unserialize($schedule_days);
	//Get the weekday number
	$today = @gmdate('w', time() + ($config['board_timezone'] * 3600) + ($config['board_dst'] * 3600));
	//Get the current board time. This prevents the user from manipulating the schedule.
	$now = @gmdate('Hi', time() + ($config['board_timezone'] * 3600) + ($config['board_dst'] * 3600));
	
	//Let's look through the days
	if (sizeof($days))
	{
		foreach ($days as $day)
		{
			//Check if time is within the set timeframe
			if ($day == $today && $schedule_start <= $now && $schedule_end >= $now)
			{
				return true;
			}
		}
	}
	elseif ($schedule_start <= $now && $schedule_end >= $now)
	{
		return true;
	}
	
	//It's not time yet
	return false;
}

/**
* Build Scheduled Group Membership array for saving
*/
function schedule_group_membership(&$sgm_ary)
{
	$weekday_chk = request_var('weekday_chk', array(0));
	$weekday_chk = serialize($weekday_chk);
	$timespan_start_hour = request_var('timespan_start_hour', '');
	$timespan_start_minute = request_var('timespan_start_minute', '');
	$timespan_start_ampm = request_var('timespan_start_ampm', '');
	$timespan_end_hour = request_var('timespan_end_hour', '');
	$timespan_end_minute = request_var('timespan_end_minute', '');
	$timespan_end_ampm = request_var('timespan_end_ampm', '');
	$start_time = 0;
	$end_time = 0;
	
	//Validate start and end times
	if (empty($timespan_start_hour) || empty($timespan_start_minute) || empty($timespan_start_ampm))
	{
		return 'START_TIME_INVALID';
	}
	elseif (empty($timespan_end_hour) || empty($timespan_end_minute) || empty($timespan_end_ampm))
	{
		return 'END_TIME_INVALID';
	}
	else
	{
		//Convert the entered times
		$start_time = schedule_group_time_convert(0, $timespan_start_hour, $timespan_start_minute, $timespan_start_ampm);
		$end_time = schedule_group_time_convert(0, $timespan_end_hour, $timespan_end_minute, $timespan_end_ampm);
		
		//The end time cannot happen before the start time so let's make sure of that
		if ($end_time < $start_time)
		{
			return 'END_NOT_BEFORE_START';
		}
		else
		{
			//We passed!  Let's build the array to return.
			$sgm_ary = array('group_schedule_days' 	=> 	$weekday_chk,
							 'group_schedule_start'	=>	$start_time,
							 'group_schedule_end'	=>	$end_time);
			
			//Now we return nothing
			return;
		}
	}
}

/**
* Convert the scheduled time into correct format
*/
function schedule_group_time_convert($schedule_time, $schedule_hour = 0, $schedule_minute = 0, $schedule_ampm = '')
{
	global $user;
	
	if ($schedule_ampm != '')
	{
		if (strtoupper($schedule_ampm) == $user->lang['TIME_AM'] && $schedule_hour == 12)
		{
			$schedule_hour = 0;
			//This is actually midnight
			if ($schedule_minute == 0)
			{
				return 12;
			}
		}
		elseif (strtoupper($schedule_ampm) == $user->lang['TIME_PM'] && $schedule_hour < 12)
		{
			$schedule_hour += 12;
		}
		
		return ($schedule_hour * 100) + $schedule_minute;
	}
	else
	{
		if ($schedule_time == 1)
		{
			return '12:00 ' . $user->lang['TIME_AM'];
		}
		else
		{
			$time = explode('.', number_format($schedule_time / 100, 2));
			$hour = (empty($time[0]) ? 12 : $time[0]);
			$hour = str_pad((($hour > 12) ? (int)$hour - 12 : $hour), 2, '0', STR_PAD_LEFT);
			$minute = $time[1];
			return $hour . ':' . $minute . ' ' . (($schedule_time >= 1200) ? $user->lang['TIME_PM'] : $user->lang['TIME_AM']);
		}
	}
}

/**
* Convert the days scheduled number with actual text
*/
function schedule_group_days_convert($schedule_days_ary)
{
	global $user;
	
	$days = array();
	
	if (!empty($schedule_days_ary))
	{
		foreach ($schedule_days_ary as $day)
		{
			switch ($day)
			{
				case '1':
					$days[] = $user->lang['MON'];
				break;
				case '2':
					$days[] = $user->lang['TUE'];
				break;
				case '3':
					$days[] = $user->lang['WED'];
				break;
				case '4':
					$days[] = $user->lang['THU'];
				break;
				case '5':
					$days[] = $user->lang['FRI'];
				break;
				case '6':
					$days[] = $user->lang['SAT'];
				break;
				case '0':
					$days[] = $user->lang['SUN'];
				break;
			}
		}
		
		return implode(", ", $days);
	}
	else
	{
		return $user->lang['DAILY'];
	}
}

/**
* Build the hours drop-down list
*/
function hours_dropdown($hour = -1)
{
	//Build hours drop-down for group scheduling
	$hours_ddl = '<option value="">--</option>';
	for ($i = 1; $i < 13; $i++)
	{
		$selected = ($i == $hour) ? ' selected="selected"' : '';
		$value = ($i < 10) ? '0' . $i : $i;
		$hours_ddl .= "\n<option value=\"$value\"$selected>$value</option>";
	}
	
	return $hours_ddl;
}

/**
* Build the minutes drop-down list
*/
function minutes_dropdown($minute = -1)
{
	//Build minutes drop-down for group scheduling
	$minutes_ddl = '<option value="">--</option>';
	for ($i = 0; $i < 60; $i += 15)
	{
		$selected = ($i == $minute) ? ' selected="selected"' : '';
		$value = ($i < 10) ? '0' . $i : $i;
		$minutes_ddl .= "\n<option value=\"$value\"$selected>$value</option>";
	}
	
	return $minutes_ddl;
}

?>